﻿using Hotline.Share.Dtos;
using Hotline.Share.Enums.FlowEngine;
using SqlSugar;

namespace Hotline.FlowEngine.Workflows;

public class WorkflowStep : StepBasicEntity
{
    /// <summary>
    /// 配置下一步节点 & 谁被选中
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true)]
    public List<StepSimple> NextSteps { get; set; } = new();

    /// <summary>
    /// 前一级节点Id(stepBox此字段为上级stepBoxId,step为上级stepId),汇总节点无此字段（因可能有多个上级来源）
    /// </summary>
    public string? PreviousId { get; set; }
    public string? PreviousCode { get; set; }

    /// <summary>
    /// 主办
    /// </summary>
    public bool IsMain { get; set; }

    /// <summary>
    /// 节点办理状态
    /// </summary>
    public EWorkflowStepStatus Status { get; set; } = EWorkflowStepStatus.WaitForAccept;


    #region 会签

    /// <summary>
    /// 会签流程生命周期上级节点Id
    /// </summary>
    public string? ParentId { get; set; }

    /// <summary>
    /// 会签流程生命周期下级节点
    /// </summary>
    [SugarColumn(IsIgnore = true)]
    public List<WorkflowStep> Steps { get; set; } = new();

    /// <summary>
    /// 发起会签生成会签Id（不发起会签节点无此字段）
    /// </summary>
    public string? StartCountersignId { get; set; }
    
    /// <summary>
    /// 会签汇总节点
    /// </summary>
    public bool IsCountersignSummaryStep { get; set; }

    /// <summary>
    /// 开启会签节点id（会签汇总节点对应会签发起节点id）
    /// </summary>
    public string StartCountersignStepId { get; set; }

    /// <summary>
    /// 会签节点
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true)]
    public List<CountersignStep> CountersignSteps { get; set; }

    /// <summary>
    /// 会签id（或外层会签的id）
    /// </summary>
    public string? CountersignId { get; set; }

    /// <summary>
    /// 当前节点发起的会签是否已汇总，冗余（未发起会签的节点此字段为null）
    /// <remarks>
    /// 发起的会签结束以后更新，如果有子会签，则所有子会签都结束以后才会更新此字段
    /// </remarks>
    /// </summary>
    public bool? IsStartedCountersignSummary { get; set; }

    /// <summary>
    /// 节点当前会签状态（区别直接办理会签和会签内非会签节点）
    /// </summary>
    public EStepCountersignStatus StepCountersignStatus { get; set; }

    #endregion

    #region Method

    /// <summary>
    /// 是否处于会签流程中
    /// </summary>
    public bool IsInCountersign() => !string.IsNullOrEmpty(CountersignId);

    /// <summary>
    /// 是否发起过会签
    /// </summary>
    public bool HasStartedCountersign() => !string.IsNullOrEmpty(StartCountersignId);

    /// <summary>
    /// 接办
    /// </summary>
    public void Accept(string userId, string? userName, string orgId, string? orgName, string? orgAreaCode,
        string? orgAreaName)
    {
        AcceptorId = userId;
        AcceptorName = userName;
        AcceptorOrgId = orgId;
        AcceptorOrgName = orgName;
        AcceptorOrgAreaCode = orgAreaCode;
        AcceptorOrgAreaName = orgAreaName;
        AcceptTime = DateTime.Now;
        Status = EWorkflowStepStatus.Handling;
    }

    /// <summary>
    /// 节点办理完成
    /// </summary>
    public void Handled(string userId, string? userName, string orgId, string? orgName, string? orgAreaCode,
        string? orgAreaName, string nextStepCode)
    {
        HandlerId = userId;
        HandlerName = userName;
        HandlerOrgId = orgId;
        HandlerOrgName = orgName;
        HandlerOrgAreaCode = orgAreaCode;
        HandlerOrgAreaName = orgAreaName;
        HandleTime = DateTime.Now;
        Status = EWorkflowStepStatus.Handled;

        if (StepType is not EStepType.End)
            NextSteps.First(d => d.Code == nextStepCode).Selected = true;
    }

    /// <summary>
    /// 检查stepBox状态，如果子节点全都办理，则stepBox办理完成
    /// </summary>
    public void CheckStepBoxStatusAndUpdate()
    {
        if (Status is not EWorkflowStepStatus.Handled && Steps.All(d => d.Status is EWorkflowStepStatus.Handled))
        {
            Status = EWorkflowStepStatus.Handled;
            HandleTime = Steps.Max(d => d.HandleTime);
        }
    }

    /// <summary>
    /// 会签结束
    /// </summary>
    public void CountersignComplete() => IsStartedCountersignSummary = true;

    /// <summary>
    /// 开启会签
    /// </summary>
    public void StartCountersign(string startCountersignId)
    {
        StartCountersignId = startCountersignId;
        IsStartedCountersignSummary = false;
    }

    /// <summary>
    /// step设置为可接办状态
    /// </summary>
    public void SetAssigned() => Status = EWorkflowStepStatus.WaitForAccept;

    /// <summary>
    /// 依据当前节点获取下一节点会签状态
    /// </summary>
    /// <returns></returns>
    public EStepCountersignStatus GetNextStepCountersignStatus()
    {
        if (HasStartedCountersign)
        {
            return EStepCountersignStatus.InCountersign;
        }

        return IsInCountersign ? EStepCountersignStatus.OuterCountersign : EStepCountersignStatus.None;
    }

    /// <summary>
    /// 重置节点,只清除办理痕迹（退回场景，将上一级节点重置等待重新办理）
    /// </summary>
    public void Reset()
    {
        SetAssigned();
        NextHandlers = new();
        NextMainHandler = null;
        NextStepCode = string.Empty;
        IsSms = default;
        Opinion = string.Empty;
        Additions = new();

        HandlerId = null;
        HandlerName = null;
        HandlerOrgId = null;
        HandlerOrgName = null;
        HandleTime = null;

        AcceptorId = null;
        AcceptorName = null;
        AcceptorOrgId = null;
        AcceptorOrgName = null;
        AcceptTime = null;
    }

    #endregion
}

public class CountersignStep
{
    public string Id { get; set; }
    public bool Completed { get; set; }
}

/*
获取开始节点下一步待选节点：
  是否属于多次实例化配置（dynamic动态实例化配置：结合办理人动态生成待办节点）
    t:无需检查是否到结束标识，所以按配置的重复实例化策略结合当前操作人生成待办节点，如：生成x级部门办理，下一接口再选具体部门
    f:开始节点配置的下一节点集合

获取开始节点所选下一步节点的具体参数：
  所选节点是否为开始节点
    t:是否属于多次实例化配置
      t:所选x级部门返回对应办理对象集合(具体部门)
      f:throw error
    f:按模板配置返回办理对象集合

获取待办节点下一步待选节点：
  是否属于多次实例化配置（无需检查definetion,直接找到当前待办节点获取参数（需要的参数冗余至待办节点上））
    t:检查是否到结束标识（检查当前办理节点的办理对象与结束标识是否一致）
      t:返回配置的下一节点(看作单次实例化，当前节点已办完)//definitionId,stepCodes，按配置走
      f:按配置的重复实例化策略结合当前操作人，生成如：{x:X级部门办理},{0:中心办理},最高级最多只能到中心，按动态策略走,动态策略枚举
  是否处于会签中 
    t:按会签策略的下一级 + 上一级（isCSEnd会签汇总节点标记）,并对上一级选项打上标记，如：{x:X级部门办理},{0:中心办理},按动态策略走，动态策略枚举
    f:按照definition(按配置走)
  返回当前节点的配置下一步

获取下一步的具体参数：
  检查模式
    按动态策略：依据当前操作人、动态类型枚举 查询对应待选部门
    按配置策略：原有实现方案

start:
  check配置是否允许当前用户发起

next: 
  check该不该当前用户办(handlers，all不需要check)
  办理
  创建后面节点
    是否处于会签中 
      t: 是否返回上一级汇总 (isCSEnd)
        t:  1.update上一级的会签完成情况[] 2.如果会签全完成，创建新的待办汇总节点?更新发起节点状态？thk
      f: 创建新的下一步节点
 */